import ctypes as ct
import numpy as np

#C definition from modal_pipe_interfaces.h
class camera_image_metadata_t(ct.Structure):
    _pack_   = 1 #struct is packed
    _fields_ = [('magic_number', ct.c_uint32),
                ('timestamp_ns', ct.c_int64),
                ('frame_id'    , ct.c_int32),
                ('width'       , ct.c_int16),
                ('height'      , ct.c_int16),
                ('size_bytes'  , ct.c_int32),
                ('stride'      , ct.c_int32),
                ('exposure_ns' , ct.c_int32),
                ('gain'        , ct.c_int16),
                ('format'      , ct.c_int16),
                ('framerate'   , ct.c_int16),
                ('reserved'    , ct.c_int16)]


CAMERA_MAGIC_NUMBER = int('0x564F584C',16)

pympa = ct.CDLL("libpympa.so")

pympa.subscribe.argtypes   = [ct.POINTER(ct.c_uint8), ct.POINTER(ct.c_uint8)]
pympa.subscribe.restype    = ct.c_int

pympa.create_pub.argtypes  = [ct.POINTER(ct.c_uint8), ct.POINTER(ct.c_uint8)]
pympa.create_pub.restype   = ct.c_int

pympa.get_image.argtypes   = [ct.POINTER(ct.c_uint8), ct.c_void_p]
pympa.get_image.restype    = ct.c_int

pympa.publish_image.argtypes = [ct.POINTER(ct.c_uint8), ct.c_void_p]
pympa.publish_image.restype  = ct.c_int

pympa.cleanup.argtypes = []
pympa.cleanup.restype  = ct.c_int

def pympa_subscribe(pipe_name, subscriber_name):
    pipe_name_np  = np.asarray((pipe_name+'\0').encode('utf-8'))
    sub_name_np   = np.asarray((subscriber_name+'\0').encode('utf-8'))
    return pympa.subscribe(pipe_name_np.ctypes.data_as(ct.POINTER(ct.c_uint8)), sub_name_np.ctypes.data_as(ct.POINTER(ct.c_uint8)))
    
def pympa_create_pub(pipe_name, publisher_name):
    pipe_name_np  = np.asarray((pipe_name+'\0').encode('utf-8'))
    pub_name_np   = np.asarray((publisher_name+'\0').encode('utf-8'))
    #print('{}'.format(''.join("{0:d} ".format(x) for x in bytes(pub_name_np))))
    return pympa.create_pub(pipe_name_np.ctypes.data_as(ct.POINTER(ct.c_uint8)), pub_name_np.ctypes.data_as(ct.POINTER(ct.c_uint8)))

def pympa_get_image(image_buffer, meta):
    return pympa.get_image(image_buffer.ctypes.data_as(ct.POINTER(ct.c_uint8)), ct.byref(meta))

def pympa_publish_image(image_buffer, meta):
    return pympa.publish_image(image_buffer.ctypes.data_as(ct.POINTER(ct.c_uint8)),ct.byref(meta))

